#include "awindow.h"

#include "text.h"
#include "button.h"

int AWindow::create(AWindow *parent,int x,int y,int width,int height,int bg,char *title)
{
  Display *dsp=aguix->getDisplay();
  Window ParentWnd;
  XTextProperty windowname,iconname;
  int scr=aguix->getScreen();
  parentawindow=parent;
  if(parent==NULL) {
    ParentWnd=RootWindow(dsp,scr);
  } else {
    ParentWnd=parent->win;
  }
  Visual *vis=DefaultVisual(dsp,scr);
  unsigned long mask=CWEventMask|CWBackPixel|CWColormap;
  XSetWindowAttributes attr;
  attr.event_mask=ExposureMask|ButtonPressMask|ButtonReleaseMask|StructureNotifyMask|KeyPressMask|KeyReleaseMask|ButtonMotionMask|PointerMotionHintMask;
  attr.background_pixel=aguix->getPixel(bg);
  attr.colormap=aguix->getColormap();
  win=XCreateWindow(dsp,ParentWnd,x,y,width,height,0,aguix->getDepth(),InputOutput,vis,mask,&attr);
  if(!win) {
    return 1;
  }
  XClassHint classhint;
  classhint.res_name=dupstring(title);
  classhint.res_class=aguix->getClassname();
  XSetClassHint(dsp,win,&classhint);
  _freesafe(classhint.res_class);
  _freesafe(classhint.res_name);
  XSetWMProtocols(dsp,win,aguix->getCloseAtom(),1);
  XSizeHints *SizeHints;
  XWMHints *WMHints;
  SizeHints=XAllocSizeHints();
  WMHints=XAllocWMHints();
  SizeHints->flags=PSize;
  SizeHints->width=width;
  SizeHints->height=height;
  WMHints->input=True;

  WMHints->window_group=aguix->getGroupWin();

  WMHints->flags=InputHint|WindowGroupHint;
  XStringListToTextProperty(&title,1,&windowname);
  XStringListToTextProperty(&title,1,&iconname);
  XSetWMName(dsp,win,&windowname);
  XSetWMIconName(dsp,win,&iconname);
  XSetWMHints(dsp,win,WMHints);
  XSetWMNormalHints(dsp,win,SizeHints);
  XStoreName(dsp,win,title);
  XFree(SizeHints);
  XFree(WMHints);
  w=width;
  h=height;
  maxw=-1;
  maxh=-1;
  minw=-1;
  minh=-1;
  XClearWindow(dsp,win);
  XSetCommand(dsp,win,aguix->getargv(),aguix->getargc());
  /* jetzt this noch in aguix anhngen */
  aguix->insertWindow(this);
  if(parent!=NULL) {
    parent->addSubAWindow(this);
  }
  XFlush(dsp);
  this->name=dupstring(title);
  this->title=dupstring(title);
  this->iconname=dupstring(title);
  this->bg=bg;
  return 0;
}

AGUIX *AWindow::getAGUIX()
{
  return aguix;
}

AWindow::AWindow(AGUIX *parent)
{
  this->aguix=parent;
  title=NULL;
  name=NULL;
  iconname=NULL;
  gui=new List();
  subwins=new List();
  subawins=new List();
  onScreen=false;
}

void AWindow::sizeChanged(int width,int height)
{
  if((width!=w)||(height!=h)) {
    AGMessage *agmsg=(AGMessage*)_allocsafe(sizeof(AGMessage));
    agmsg->type=AG_SIZECHANGED;
    agmsg->size.window=win;
    agmsg->size.neww=width;
    agmsg->size.newh=height;
    aguix->putAGMsg(agmsg);
    w=width;
    h=height;
  }
}

void AWindow::redraw()
{
  aguix->SetWindowBG(win,bg);
  Display *dsp=aguix->getDisplay();
  int scr=aguix->getScreen();
  Visual *vis=DefaultVisual(dsp,scr);
  unsigned long mask=CWEventMask|CWBackPixel;
  XSetWindowAttributes attr;
  attr.event_mask=ExposureMask|ButtonPressMask|ButtonReleaseMask|StructureNotifyMask|KeyPressMask|KeyReleaseMask|ButtonMotionMask;
  attr.background_pixel=aguix->getPixel(bg);
  Window twin=XCreateWindow(dsp,win,0,0,w,h,0,aguix->getDepth(),InputOutput,vis,mask,&attr);
  if(!twin) {
    return;
  }
  XMapRaised(dsp,twin);
  XDestroyWindow(dsp,twin);
}

Window AWindow::getWindow()
{
  return win;
}

void AWindow::close()
{
  int id=gui->initEnum();
  GUIElement *telem=(GUIElement*)gui->getFirstElement(id);
  while(telem!=NULL) {
    delete telem;
    gui->removeElementAt(0);
    telem=(GUIElement*)gui->getFirstElement(id);
  }
  gui->closeEnum(id);
  Display *dsp=aguix->getDisplay();
  aguix->removeWindow(this);
  XUnmapWindow(dsp,win);
  XDestroyWindow(dsp,win);
  aguix->Flush();
  if(parentawindow!=NULL) parentawindow->removeSubAWindow(this);
  delete this;
}

AWindow::~AWindow()
{
  if(title!=NULL) _freesafe(title);
  if(name!=NULL) _freesafe(name);
  if(iconname!=NULL) _freesafe(iconname);
  int id=gui->initEnum();
  GUIElement *te=(GUIElement*)gui->getFirstElement(id);
  while(te!=NULL) {
    delete te;
    te=(GUIElement*)gui->getNextElement(id);
  }
  gui->closeEnum(id);
  delete gui;
  delete subwins;
  delete subawins;
}

int AWindow::getBG()
{
  return bg;
}

void AWindow::setBG(int bg)
{
  this->bg=bg;
}

void AWindow::ReactMessage(Message *msg)
{
  if((msg->window)!=win) return;
  // diese Message interressiert uns
}

GUIElement *AWindow::add(GUIElement *new_element)
{
  gui->addElement(new_element);
  new_element->setParent(this);
  return new_element;
}

void AWindow::setMaxSize(int mwidth,int mheight)
{
  XSizeHints *SizeHints;
  Display *dsp=aguix->getDisplay();
  SizeHints=XAllocSizeHints();
  if(maxw>=0) {
    SizeHints->flags=PMaxSize|PMinSize;
    SizeHints->min_width=minw;
    SizeHints->min_height=minh;
  } else {
    SizeHints->flags=PMaxSize;
  }
  SizeHints->max_width=mwidth;
  SizeHints->max_height=mheight;
  maxw=mwidth;
  maxh=mheight;
  XSetWMNormalHints(dsp,win,SizeHints);
  XFree(SizeHints);
}

void AWindow::setMinSize(int mwidth,int mheight)
{
  XSizeHints *SizeHints;
  Display *dsp=aguix->getDisplay();
  SizeHints=XAllocSizeHints();
  if(maxw>=0) {
    SizeHints->flags=PMaxSize|PMinSize;
    SizeHints->max_width=maxw;
    SizeHints->max_height=maxh;
  } else {
    SizeHints->flags=PMinSize;
  }
  SizeHints->min_width=mwidth;
  SizeHints->min_height=mheight;
  minw=mwidth;
  minh=mheight;
  XSetWMNormalHints(dsp,win,SizeHints);
  XFree(SizeHints);
}

void AWindow::show()
{
  if(onScreen==true) return;
  Display *dsp=aguix->getDisplay();
  XMapWindow(dsp,win);
  onScreen=true;
}

void AWindow::hide()
{
  if(onScreen==false) return;
  Display *dsp=aguix->getDisplay();
  XUnmapWindow(dsp,win);
  onScreen=false;
}

void AWindow::hide(Window subwin)
{
  if(hasSubWin(subwin)==true) {
    Display *dsp=aguix->getDisplay();
    XUnmapWindow(dsp,subwin);
  }
}

void AWindow::show(Window subwin)
{
  if(hasSubWin(subwin)==true) {
    Display *dsp=aguix->getDisplay();
    XMapWindow(dsp,subwin);
  }
}

char *AWindow::getTitle()
{
  return title;
}

void AWindow::setTitle(const char *newtitle)
{
  _freesafe(title);
  title=dupstring(newtitle);
  Display *dsp=aguix->getDisplay();
  XClassHint classhint;
  classhint.res_name=dupstring(title);
  classhint.res_class=aguix->getClassname();
  XSetClassHint(dsp,win,&classhint);
  _freesafe(classhint.res_class);
  _freesafe(classhint.res_name);
  XTextProperty windowname,iconname;
  XStringListToTextProperty(&title,1,&windowname);
  XStringListToTextProperty(&title,1,&iconname);
  XSetWMName(dsp,win,&windowname);
  XSetWMIconName(dsp,win,&iconname);
  XStoreName(dsp,win,title);
}

void AWindow::resizeSubWin(Window win,int w,int h)
{
  Display *dsp=aguix->getDisplay();
  XResizeWindow(dsp,win,w,h);
}

void AWindow::moveSubWin(Window win,int x,int y)
{
  Display *dsp=aguix->getDisplay();
  XMoveWindow(dsp,win,x,y);
}

bool AWindow::handleMessage(XEvent *E,Message *msg)
{
//  int pos;
  bool returnvalue=false;
  if(msg->window==win) {
    if(msg->type==ConfigureNotify) {
      sizeChanged(E->xconfigure.width,E->xconfigure.height);
    }
  }
//  pos=subwins->getIndex((void*)(msg->window));
//  if((pos>=0)||(msg->window==win)) {
    // Alle Element bekommen Nachricht
    int id=gui->initEnum();
    GUIElement *guie=(GUIElement*)gui->getFirstElement(id);
    while((guie!=NULL)&&(returnvalue==false)) {
      returnvalue=guie->handleMessage(E,msg);
      guie=(GUIElement*)gui->getNextElement(id);
    }
    gui->closeEnum(id);
//  }
  return returnvalue;
}

bool AWindow::hasSubWin(Window win)
{
  int pos=subwins->getIndex((void*)win);
  if(pos==-1) return false;
  else return true;
}

Window AWindow::getSubWindow(Window parent,int x,int y,int w,int h)
{
  Window newwin,pwin;
  Display *dsp=aguix->getDisplay();
  int scr=aguix->getScreen();
  Visual *vis=DefaultVisual(dsp,scr);
  unsigned long mask=CWEventMask|CWBackPixel|CWColormap;

  XSetWindowAttributes attr;
  attr.event_mask=ExposureMask|ButtonPressMask|ButtonReleaseMask|StructureNotifyMask|KeyPressMask|KeyReleaseMask|ButtonMotionMask|EnterWindowMask|LeaveWindowMask|PointerMotionHintMask;
  attr.background_pixel=aguix->getPixel(bg);
  attr.colormap=aguix->getColormap();
  if(parent!=0) pwin=parent; else pwin=win;
  newwin=XCreateWindow(dsp,pwin,x,y,w,h,0,aguix->getDepth(),InputOutput,vis,mask,&attr);
  if(!newwin) {
    return newwin;
  }
  XMapRaised(dsp,newwin);
  XClearWindow(dsp,newwin);
  subwins->addElement((void*)newwin);
  return newwin;
}

void AWindow::removeSubWin(Window subwin)
{
  if(hasSubWin(subwin)==true) {
    Display *dsp=aguix->getDisplay();
    XDestroyWindow(dsp,subwin);
    subwins->removeElement((void*)subwin);
  }
}

void AWindow::addSubAWindow(AWindow *child)
{
  subawins->addElement(child);
}

void AWindow::removeSubAWindow(AWindow *child)
{
  subawins->removeElement(child);
}

bool AWindow::isParent(Window childwin,bool direct)
{
  int id=subawins->initEnum();
  bool found=false;
  if(direct==false) {
    AWindow *tawin=(AWindow*)subawins->getFirstElement(id);
    while(tawin!=NULL) {
      if(tawin->isParent(childwin,true)==true) found=true;
      tawin=(AWindow*)subawins->getNextElement(id);
    }
  }
  if(hasSubWin(childwin)==true) found=true;
  if(childwin==win) found=true;
  subawins->closeEnum(id);
  return found;
}

bool AWindow::remove(GUIElement *elem)
{
  bool returnvalue=false;
  int pos=gui->getIndex(elem);
  if(pos>=0) {
    returnvalue=true;
    gui->removeElementAt(pos);
    delete elem;
  }
  return returnvalue;
}

void AWindow::move(int nx,int ny)
{
  Display *dsp=aguix->getDisplay();
  XMoveWindow(dsp,win,nx,ny);
  x=nx;
  y=ny;
}

void AWindow::resize(int nw,int nh)
{
  Display *dsp=aguix->getDisplay();
  XResizeWindow(dsp,win,nw,nh);
  w=nw;
  h=nh;
}

GUIElement *AWindow::findGUIElement(Window child)
{
  int id;
  id=gui->initEnum();
  GUIElement *guiel=(GUIElement*)gui->getFirstElement(id);
  while(guiel!=NULL) {
    if(guiel->isParent(child)==true) break;
    guiel=(GUIElement*)gui->getNextElement(id);
  }
  gui->closeEnum(id);
  return guiel;
}
