#include "enterpathop.h"
#include "listermode.h"
#include "normalmode.h"
#include "worker.h"

const char *EnterPathOp::name="EnterPathOp";

EnterPathOp::EnterPathOp(Worker *worker):FunctionProto(worker)
{
  enterpathside=ENTERPATHOP_THISSIDE;
}

EnterPathOp::~EnterPathOp()
{
}

EnterPathOp*
EnterPathOp::duplicate()
{
  EnterPathOp *ta=new EnterPathOp(worker);
  ta->enterpathside=enterpathside;
  return ta;
}

bool
EnterPathOp::isName(const char *str)
{
  if(strcmp(str,name)==0) return true; else return false;
}

const char *
EnterPathOp::getName()
{
  return name;
}

int
EnterPathOp::run(ActionMessage *msg)
{
  ListerMode *lm1;
  am=msg;
  if(msg->mode!=msg->AM_MODE_DNDACTION) {
    Lister *l1=worker->getActiveLister();
    if(l1!=NULL) {
      startlister=l1;
      lm1=startlister->getActiveMode();
      if(lm1!=NULL)
        if(lm1->isType("NormalMode")==true)
          normalmodeenterpath();
    }
  }
  return 0;
}

bool
EnterPathOp::save(Datei *fh)
{
  int size=Datei::getUCharSize();
  fh->putInt(size);

  switch(enterpathside) {
    case ENTERPATHOP_OTHERSIDE:
      fh->putUChar(1);
      break;
    case ENTERPATHOP_LEFTSIDE:
      fh->putUChar(2);
      break;
    case ENTERPATHOP_RIGHTSIDE:
      fh->putUChar(3);
      break;
    default:
      fh->putUChar(0);
  }
  return true;
}

int
EnterPathOp::load(Datei *fh)
{
  int chunksize=fh->getInt();
  int cs=Datei::getUCharSize();
  unsigned char c1;
  if(chunksize>=cs) {
    c1=fh->getUChar();
    switch(c1) {
      case 1:
        enterpathside=ENTERPATHOP_OTHERSIDE;
        break;
      case 2:
        enterpathside=ENTERPATHOP_LEFTSIDE;
        break;
      case 3:
        enterpathside=ENTERPATHOP_RIGHTSIDE;
        break;
      default:
        enterpathside=ENTERPATHOP_THISSIDE;
        break;
    }
    chunksize-=cs;
  }
  while(chunksize>0) {
    fh->getUChar();
    chunksize--;
  }
  return true;
}

const char *
EnterPathOp::getDescription()
{
  return catalog.getLocaleCom(34);
}

void
EnterPathOp::normalmodeenterpath()
{
  NormalMode *nm1=NULL;
  ListerMode *lm1=NULL;
  Lister *ll,*lr,*tl;
  
  ll=worker->getLister(0);
  lr=worker->getLister(1);
  
  tl=NULL;
  switch(enterpathside) {
    case ENTERPATHOP_OTHERSIDE:
      if(startlister==ll) tl=lr;
      else tl=ll;
      break;
    case ENTERPATHOP_LEFTSIDE:
      tl=ll;
      break;
    case ENTERPATHOP_RIGHTSIDE:
      tl=lr;
      break;
    default:
      tl=startlister;
      break;
  }
  if(tl!=NULL) {
    lm1=tl->getActiveMode();
    if(lm1!=NULL) {
      if(lm1->isType("NormalMode")==true) {
        nm1=(NormalMode*)lm1;
        nm1->enterpath();
      }
    }
  }
}

int
EnterPathOp::configure()
{
  AGUIX *aguix=worker->getAGUIX();
  Button *okb,*cb;
  AWindow *win;
  CycleButton *rcyb;
  Text *ttext;
  int tw,w,h,x,y,t1,t2;
  AGMessage *msg;
  int endmode=-1;
  char *tstr;
  
  w=h=10;
  x=y=5;
  win=new AWindow(aguix);

  tstr=(char*)_allocsafe(strlen(catalog.getLocale(293))+strlen(catalog.getLocaleCom(34))+1);
  sprintf(tstr,catalog.getLocale(293),catalog.getLocaleCom(34));
  win->create(NULL,10,10,w,h,0,tstr);
  _freesafe(tstr);

  ttext=(Text*)win->add(new Text(aguix,x,y,catalog.getLocale(323),1));
  x+=ttext->getWidth()+5;
  rcyb=(CycleButton*)win->add(new CycleButton(aguix,x,y,100,1,0,0));
  rcyb->addOption(catalog.getLocale(324));
  rcyb->addOption(catalog.getLocale(325));
  rcyb->addOption(catalog.getLocale(326));
  rcyb->addOption(catalog.getLocale(327));
  rcyb->resize(rcyb->getMaxSize(),rcyb->getHeight());
  switch(enterpathside) {
    case ENTERPATHOP_OTHERSIDE:
      rcyb->setOption(1);
      break;
    case ENTERPATHOP_LEFTSIDE:
      rcyb->setOption(2);
      break;
    case ENTERPATHOP_RIGHTSIDE:
      rcyb->setOption(3);
      break;
    default:
      rcyb->setOption(0);
      break;
  }
  
  y+=rcyb->getHeight()+5;
  tw=x+rcyb->getWidth()+5;
  if(tw>w) w=tw;
  x=5;

  t1=(strlen(catalog.getLocale(11))+2);
  t1*=aguix->getCharWidth();
  t2=(strlen(catalog.getLocale(8))+2);
  t2*=aguix->getCharWidth();
  tw=5+t1+5+t2+5;
  if(tw>w) w=tw;
  okb=(Button*)win->add(new Button(aguix,
                                   5,
                                   y,
                                   t1,
                                   catalog.getLocale(11),
                                   1,
                                   0,
                                   0));
  cb=(Button*)win->add(new Button(aguix,
                                  w-5-t2,
                                  y,
                                  t2,
                                  catalog.getLocale(8),
                                  1,
                                  0,
                                  0));
  y+=okb->getHeight()+5;
  
  h=y;
  win->resize(w,h);
  win->setMaxSize(w,h);
  win->setMinSize(w,h);
  win->show();
  for(;endmode==-1;) {
    msg=aguix->WaitMessage(win);
    if(msg!=NULL) {
      switch(msg->type) {
        case AG_CLOSEWINDOW:
          if(msg->closewindow.window==win->getWindow()) endmode=1;
          break;
        case AG_BUTTONCLICKED:
          if(msg->button.button==okb) endmode=0;
          else if(msg->button.button==cb) endmode=1;
          break;
      }
      aguix->ReplyMessage(msg);
    }
  }
  
  if(endmode==0) {
    // ok
    switch(rcyb->getSelectedOption()) {
      case 1:
        enterpathside=ENTERPATHOP_OTHERSIDE;
        break;
      case 2:
        enterpathside=ENTERPATHOP_LEFTSIDE;
        break;
      case 3:
        enterpathside=ENTERPATHOP_RIGHTSIDE;
        break;
      default:
        enterpathside=ENTERPATHOP_THISSIDE;
        break;
    }
  }
  
  win->close();

  return endmode;
}

void EnterPathOp::setSide(enterpath_t nv)
{
  enterpathside=nv;
}

